#!/usr/local/bin/python
#
# $Id: CMFFeedApp.py,v 1.17 2002/03/16 19:05:50 dhellmann Exp $
#
# Copyright 2001 Doug Hellmann.
#
#
#                         All Rights Reserved
#
# Permission to use, copy, modify, and distribute this software and
# its documentation for any purpose and without fee is hereby
# granted, provided that the above copyright notice appear in all
# copies and that both that copyright notice and this permission
# notice appear in supporting documentation, and that the name of Doug
# Hellmann not be used in advertising or publicity pertaining to
# distribution of the software without specific, written prior
# permission.
#
# DOUG HELLMANN DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
# INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN
# NO EVENT SHALL DOUG HELLMANN BE LIABLE FOR ANY SPECIAL, INDIRECT OR
# CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
# OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
# NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
# CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#

"""Base class for apps that feed data into a CMF portal.

"""

__rcs_info__ = {
    #
    #  Creation Information
    #
    'module_name'  : '$RCSfile: CMFFeedApp.py,v $',
    'rcs_id'       : '$Id: CMFFeedApp.py,v 1.17 2002/03/16 19:05:50 dhellmann Exp $',
    'creator'      : 'Doug Hellmann <doug@hellfly.net>',
    'project'      : 'CMF',
    'created'      : 'Sat, 10-Nov-2001 06:22:54 EST',

    #
    #  Current Information
    #
    'author'       : '$Author: dhellmann $',
    'version'      : '$Revision: 1.17 $',
    'date'         : '$Date: 2002/03/16 19:05:50 $',
}

#
# Import system modules
#
import os
import string
try:
    from cStringIO import StringIO
except:
    from StringIO import StringIO
import sys
import urllib2
from UserDict import UserDict

#
# Import Local modules
#
from CommandLineApp import CommandLineApp
from ZctlApp import ZctlApp, INSTANCE_HOME
import DateTime

#
# Module
#

class CMFFeedApp(CommandLineApp, ZctlApp):

    default_config_values = {
        'member_name':'newsfeed',
        'portal_name':'Portal',
        'test_mode':0,
        }

    num_new_articles = 0

    config_file_base = 'agent.conf'
    
    def readConfigFile(self):
        """Read the configuration file.

        Looks for INSTANCE_HOME/<class>.conf.
        """
        #config_file_base = '%s.conf' % self.__class__.__name__.lower()
        config_file_base = self.config_file_base
        config_file = os.path.join(INSTANCE_HOME, config_file_base)
        try:
            config_instructions = open(config_file, 'rt').read()
            self.statusMessage('Getting configuration from %s' % config_file, 2)
            del config_instructions
        except IOError, msg:
            self.statusMessage(str(msg), 2)
            self.statusMessage('Using default configuration values', 2)
            for name, value in self.default_config_values.items():
                setattr(self, name, value)
        else:
            tmp_globals = {}
            tmp_locals = {}
            execfile(config_file, tmp_globals, tmp_locals)
            for name, default_value in self.default_config_values.items():
                setattr(self, name, tmp_locals.get(name, default_value))
        return

    def appInit(self):
        self.readConfigFile()
        return

    def optionHandler_t(self):
        """Turn on test mode.

        Test mode disables committing changes and
        prints the XML content of each RDF/RSS feed
        retrieved.
        """
        self.test_mode = 1
        return

    def getObjFromPath(self, root, path):
        """Start at root, and get the object identified by path.

        The root should be either a ZODB context or an object
        from a ZODB context.

        The path should be a string with object names separated
        by periods.
        """
        path_parts = path.split('.')
        obj = root
        for p in path_parts:
            obj = getattr(obj, p)
        return obj

    #
    # Clean up characters by replacing any char in x[1] with x[0]
    #
    _replace_table = [
        ('_', '/:%$ \t'),
        ('-', '=+!'),
        (',', '.&@?*<>#;'),
        ]

    _delete_chars = '\'"'
    _replace_chars = ''.join([ x[1] for x in _replace_table ])
    _replace_by = ''.join([ (x[0] * len(x[1])) for x in _replace_table ])
    _url_to_id_trans = string.maketrans( _replace_chars, _replace_by )
    
    def urlToId(self, url):
        id = str(url)
        id = id.translate(self._url_to_id_trans, self._delete_chars)
        while id and id[0] == '_':
            id=id[1:]
        if not id:
            raise ValueError('Could not translate "%s" to a valid id.' % url)
        return id

    def fixUnicodeString(self, s):
        try:
            new_s=str(s)
        except UnicodeError:
            new_s = ""
            valid_range = range(128)
            for c in s:
                if ord(c) not in valid_range:
                    c = " "
                new_s = new_s + c
        return new_s

    def replaceEntityReferences(self, text):
        """Replaces entity references for some characters with those
        characters.
        """
        text = text.replace('&amp;#', '&#') # &#X; was converted to &amp;#X;
        return text

    def fixTextString(self, text):
        text = self.fixUnicodeString(text)
        text = self.replaceEntityReferences(text)
        return text

    def addNewsItem(self, folder, id, title, link, description, subject=''):
        
        title = self.fixTextString(title)
        description = self.fixTextString(description)
        
        self.statusMessage('News item %s' % id, 3)
        self.statusMessage('  %s' % title, 3)
        self.statusMessage('  %s' % link, 3)
        self.statusMessage('  %s' % subject, 3)
        
        try:

            #
            # Look for this article, assuming the id
            # value will be unique (because it came
            # from the URL).
            #
            existing = folder.portal_catalog.searchResults(
                meta_type='News Item',
                id=id,
                Creator=self.member_name,
                )
                
            #
            # Figure out what to do with the article
            #
            if not existing:
                #
                # We have a new item, store it.
                #
                self.statusMessage('News Item: %s' % title)
                self.statusMessage('  %s' % id)
                self.statusMessage('\n%s\n' % description)

                #
                # Get the destination folder
                #
                today = DateTime.DateTime()
                destination_folder_name = today.strftime('%Y_%m_%d')
                try:
                    destination_folder = getattr(folder,
                                                 destination_folder_name)
                except AttributeError:
                    self.statusMessage(
                        'Creating new destination folder %s' % \
                        destination_folder_name,
                        2)
                    folder.invokeFactory(
                        type_name='Folder',
                        id=destination_folder_name,
                        title='Items for %s' % today.strftime('%Y/%m/%d'),
                        )
                    destination_folder = getattr(folder,
                                                 destination_folder_name)
                
                # add the link
                title = '%s: %s' % (folder.id, title)
                destination_folder.invokeFactory(
                    type_name='News Item',
                    id=id,
                    title=title)
                new_link = getattr(destination_folder, id)
                if title:
                    text='<p>%s</p>\n<p><a href="%s">More</a></p>' \
                                 % (title, link)
                else:
                    text='<p><a href="%s">More</a></p>' \
                                 % link

                # Set properties of link
                new_link._edit(
                    text=text,
                    description=description,
                    text_format='html',
                    )
                new_link.setTitle(title)
                new_link.title = title
                new_link.setSubject(subject)
                new_link.setContributors( (self.member_name,) )
                new_link.setFormat('text/html')
                new_link.reindexObject()
                
                # mark as reviewed
                workflows = destination_folder.portal_workflow.doActionFor(
                    new_link, 
                    'publish',
                    comment='Automatically published by getnews.py',
                    )
                self.num_new_articles = self.num_new_articles + 1
                
            else:
                #
                # We already have the link
                #
                self.statusMessage('Duplicate Item: %s' % title, 2)
                self.statusMessage('  %s' % id, 2)

                # debug
#                 print
#                 if existing:
#                     for e in existing:
#                         print '  %s' % e.id
#                         print '    %s' % e.Title
                
        except UnicodeError, err:
            self.statusMessage('Error Item: %s' % title, 2)
            self.statusMessage('  %s' % id, 2)
            self.statusMessage('  unicode error', 2)
        return
    
    def loadDataForFolder(self, folder):
        "Do whatever this app needs to do with the provided folder."
        raise RuntimeError('Not implemented by %s' % self.__class__.__name__)

    def makeRequest(self):
        """From Zope.../lib/python/Testing/makerequest.py
        """
        from os import environ
        from sys import stdin, stdout
        from ZPublisher.HTTPRequest import HTTPRequest
        from ZPublisher.HTTPResponse import HTTPResponse
        from ZPublisher.BaseRequest import RequestContainer
        
        resp = HTTPResponse(stdout=stdout)
        environ['SERVER_NAME']='www.hellfly.net'
        environ['SERVER_PORT']='80'
        environ['REQUEST_METHOD'] = 'GET'
        req = HTTPRequest(stdin, environ, resp)
        
        req['AUTHENTICATED_USER'] = self.member_name
        req['SERVER_URL']= 'http://www.hellfly.net'
        req._script = ['getnews']
        
        return self.app.__of__(RequestContainer(REQUEST = req))
    
    def main(self, *args):
        self.app = self.makeRequest()
        
        self.statusMessage('Getting handle to portal...', 2)
        portal = self.getObjFromPath(self.app, self.portal_name)

        self.statusMessage('Getting member...', 2)
        try:
            acl_users = portal.acl_users
        except AttributeError:
            # huh?
            self.statusMessage('Could not get acl_users folder')
        else:
            authenticated_user = acl_users.getUser(self.member_name)
            if not authenticated_user:
                raise RuntimeError(
                    'No user "%s" found in the Portal.' % self.member_name
                    )
            authenticated_user = authenticated_user.__of__(acl_users)
            self.app.REQUEST['AUTHENTICATED_USER'] = authenticated_user
            from AccessControl import getSecurityManager
            sm = getSecurityManager()
            sm_context = sm._context
            sm._context.user = authenticated_user

        self.statusMessage('Getting member folder...', 2)
        member_folder = self.getObjFromPath(
            portal, 'Members.%s' % self.member_name)

        self.statusMessage('Finding data sources...', 2)
        folders = member_folder.contentValues('Portal Folder')
        for folder in folders:
            self.statusMessage('', 2)
            self.loadDataForFolder(folder)

        del sm_context.user
        if not self.test_mode:
            max_attempts = 3
            for attempt in range(max_attempts):
                try:
                    self.statusMessage('Commit trial %d' % attempt, 2)
                    get_transaction().commit()
                except Exception, msg:
                    self.statusMessage(
                        'Commit trial %d failed with error "%s"' % (
                        attempt, str(msg))
                                       )
                    if attempt in range(max_attempts):
                        self.statusMessage('Retrying')
                    else:
                        raise
                else:
                  break
        else:
            self.statusMessage('Test mode, commit skipped.')

        if self.num_new_articles:
            show_count = 1
        else:
            show_count = 2
        self.statusMessage('Added %d new articles.' % self.num_new_articles,
                           show_count)
            
        return
    
